@@ -12,8 +12,9 @@
 #
 ##############################################################################
 
-import os, re, shutil, sys
+import os, os.path, re, shutil, sys
 import zc.buildout
+import zc.buildout.easy_install
 import zc.recipe.egg
 
 class Recipe:
@@ -53,8 +54,11 @@ class Recipe:
             mkzopeinstance = '"%s"' % mkzopeinstance
 
         assert os.spawnl(
-            os.P_WAIT, options['executable'], options['executable'],
-            mkzopeinstance, '-d', location, '-u', options['user'],
+            os.P_WAIT, options['executable'], 
+            zc.buildout.easy_install._safe_arg(options['executable']),
+            mkzopeinstance, '-d', 
+            zc.buildout.easy_install._safe_arg(location), 
+            '-u', options['user'],
             ) == 0
 
         try:
@@ -75,7 +79,8 @@ class Recipe:
             self.build_package_includes()
         except:
             # clean up
-            shutil.rmtree(location)
+            if os.path.exists(location):
+                shutil.rmtree(location)
             raise
 
         return location
@@ -157,6 +162,11 @@ class Recipe:
             if port_base:
                 port_base = 'port-base %s' % port_base
             http_address = options.get('http-address', '8080')
+            http_fast_listen = options.get('http-fast-listen', None)
+            if http_fast_listen is None:
+                http_fast_listen = ''
+            else:
+                http_fast_listen = http_fast_listen_template % http_fast_listen
             ftp_address = options.get('ftp-address', '')
             if ftp_address:
                 ftp_address = ftp_server_template % ftp_address
@@ -191,6 +201,18 @@ class Recipe:
                                                  for env_var in env_vars])
                 environment_vars = environment_template % environment_vars
             
+            deprecation_warnings = options.get('deprecation-warnings', '')
+            if deprecation_warnings:
+                if deprecation_warnings.lower() in ('off', 'disable', 'false'):
+                    deprecation_warnings = 'ignore'
+                elif deprecation_warnings.lower() in ('enable', 'on', 'true'):
+                    deprecation_warnings = 'default'
+                deprecation_warnings = '\n'.join((
+                    "<warnfilter>",
+                    "  action %s" % deprecation_warnings,
+                    "  category exceptions.DeprecationWarning",
+                    "</warnfilter>"))
+
             zope_conf_additional = options.get('zope-conf-additional', '')
 
             event_log_level = options.get('event-log-level', 'INFO')
@@ -363,6 +385,7 @@ class Recipe:
                                         storage_snippet = storage_snippet.strip(),
                                         port_base = port_base,
                                         http_address = http_address,
+                                        http_fast_listen = http_fast_listen,
                                         ftp_address = ftp_address,
                                         webdav_address = webdav_address,
                                         zserver_threads = zserver_threads,
@@ -372,6 +395,7 @@ class Recipe:
                                         pid_file = pid_file,
                                         lock_file = lock_file,
                                         environment_vars = environment_vars,
+                                        deprecation_warnings = deprecation_warnings,
                                         zope_conf_additional = zope_conf_additional,)
 
         zope_conf_path = os.path.join(location, 'etc', 'zope.conf')
@@ -417,8 +441,8 @@ class Recipe:
             # everything was turned into eggs we exceeded the
             # environment maximum size for cmd.exe.
             script = script.replace(
-                "ZOPE_RUN = r'%s\\Zope2\\Startup\\run.py' % SOFTWARE_HOME",
-                "ZOPE_RUN = r'%s\\bin\\servicewrapper.py' % INSTANCE_HOME"
+                "ZOPE_RUN=%SOFTWARE_HOME%\\Zope2\\Startup\\run.py",
+                "ZOPE_RUN=%INSTANCE_HOME%\\bin\\servicewrapper.py"
                 )
             f = open(script_path, 'w')
             f.write(script)
@@ -585,6 +609,7 @@ rel_storage_template="""
         </%(type)s>
     </relstorage>
 """
+
 blob_storage_template="""
     # Blob-enabled FileStorage database
     <blobstorage>
@@ -640,6 +665,12 @@ zodb_temporary_storage_template="""
 </zodb_db>
 """.strip()
 
+http_fast_listen_template="""\
+  # Set to off to defer opening of the HTTP socket until the end of the
+  # startup phase:
+  fast-listen %s
+""".rstrip()
+
 event_logfile = """
   <logfile>
     path %(event_logfile)s
@@ -691,6 +722,7 @@ verbose-security %(verbose_security)s
 %(zserver_threads)s
 %(zeo_client_name)s
 %(environment_vars)s
+%(deprecation_warnings)s
 
 <eventlog>
   level %(event_log_level)s
@@ -708,6 +740,7 @@ verbose-security %(verbose_security)s
   # force-connection-close on
   # You can also use the WSGI interface between ZServer and ZPublisher:
   # use-wsgi on
+%(http_fast_listen)s
 </http-server>
 
 %(ftp_address)s
